from shiva.resources.base import ArtistResource, AlbumResource, TrackResource
from shiva.resources.static import AboutResource, ClientResource
from shiva.resources.dynamic import (ConvertResource, LyricsResource,
                                     RandomResource, ShowsResource,
                                     WhatsNewResource)
